@@ -28,6 +28,7 @@
 #include <QtNetwork/QNetworkRequest>
 #include <QtCore/QProcess>
 #include <QtCore/QTimer>
+#include <QtCore/QFileInfo>
 #include <QtGui/QMessageBox>
 
 #include "GeoSceneDocument.h"
@@ -77,6 +78,7 @@ ControlView::ControlView( QWidget *parent )
 
     connect( m_control, SIGNAL( showMapWizard() ), this, SIGNAL( showMapWizard() ) );
     connect( m_control, SIGNAL( showUploadDialog() ), this, SIGNAL( showUploadDialog() ) );
+    connect( m_control, SIGNAL( searchFinished() ), this, SIGNAL( searchFinished() ) );
 }
 
 ControlView::~ControlView()
@@ -86,7 +88,7 @@ ControlView::~ControlView()
 
 QString ControlView::applicationVersion()
 {
-    return "1.2.85 (1.3 Beta 2)";
+    return "1.5.5 (stable version)";
 }
 
 void ControlView::zoomIn()
@@ -380,7 +382,7 @@ void ControlView::printDrivingInstructions( QTextDocument &document, QString &te
     text += "<tr><th>No.</th><th>Distance</th><th>Instruction</th></tr>";
     for ( int i=0; i<routingModel->rowCount(); ++i ) {
         QModelIndex index = routingModel->index(i, 0);
-        GeoDataCoordinates coordinates = qVariantValue<GeoDataCoordinates>( index.data( MarblePlacemarkModel::CoordinateRole ) );
+        GeoDataCoordinates coordinates = qVariantValue<GeoDataCoordinates>( index.data( RoutingModel::CoordinateRole ) );
         GeoDataLineString accumulator;
         for (int k=0; k<total.size(); ++k) {
             accumulator << total.at(k);
@@ -498,8 +500,8 @@ void ControlView::synchronizeWithExternalMapEditor( const QString &application,
         localEventLoop.exec();
     } else {
         // The local server is not alive. Start the application
-        QString applicationArgument = argument.arg( north, 0, 'f', 8 ).arg( east, 0, 'f', 8 );
-        applicationArgument = applicationArgument.arg( south, 0, 'f', 8 ).arg( west, 0, 'f', 8 );
+        QString applicationArgument = argument.arg( south, 0, 'f', 8 ).arg( east, 0, 'f', 8 );
+        applicationArgument = applicationArgument.arg( north, 0, 'f', 8 ).arg( west, 0, 'f', 8 );
         mDebug() << "No local server found. Launching " << application << " with argument " << applicationArgument;
         if ( !QProcess::startDetached( application, QStringList() << applicationArgument ) ) {
             QString text = tr( "Unable to start the external editor. Check that %1 is installed or choose a different external editor in the settings dialog." );
@@ -514,6 +516,11 @@ void ControlView::setExternalMapEditor( const QString &editor )
     m_externalEditor = editor;
 }
 
+void ControlView::search(const QString &searchTerm, SearchMode searchMode )
+{
+    m_control->search( searchTerm, searchMode );
+}
+
 QString ControlView::externalMapEditor() const
 {
     return m_externalEditor;
@@ -531,7 +538,12 @@ bool ControlView::setSideBarState( const QByteArray &state )
 
 void ControlView::addGeoDataFile( QString filename )
 {
-    m_marbleWidget->model()->addGeoDataFile( filename );
+    QFileInfo const file( filename );
+    if ( file.exists() ) {
+        m_marbleWidget->model()->addGeoDataFile( file.absoluteFilePath() );
+    } else {
+        qWarning() << "File" << filename << "does not exist, cannot open it.";
+    }
 }
 
 }
