@@ -37,13 +37,13 @@
 #include <QtGui/QHBoxLayout>
 #include <QtGui/QScrollArea>
 #include <QtGui/QClipboard>
-
+#include <QtGui/QShortcut>
 #include <QtNetwork/QNetworkProxy>
 
+#include "SearchInputWidget.h"
 #include "EditBookmarkDialog.h"
 #include "BookmarkManagerDialog.h"
 #include "CurrentLocationWidget.h"
-//#include "EditBookmarkDialog.h"
 #include "MapViewWidget.h"
 #include "MarbleDirs.h"
 #include "MarbleAboutDialog.h"
@@ -64,12 +64,14 @@
 #include "routing/RoutingProfilesModel.h"
 #include "routing/RoutingWidget.h"
 #include "routing/RouteRequest.h"
+#include "ParseRunnerPlugin.h"
 #include "PositionTracking.h"
 #include "PositionProviderPlugin.h"
 #include "PluginManager.h"
 #include "MapWizard.h"
 #include "StackableWindow.h"
 #include "GoToDialog.h"
+#include "MarbleWidgetInputHandler.h"
 
 // For zoom buttons on Maemo
 #ifdef Q_WS_MAEMO_5
@@ -82,6 +84,7 @@ namespace
 {
     const char* POSITION_STRING = "Position:";
     const char* DISTANCE_STRING = "Altitude:";
+    const char* ZOOM_STRING = "Zoom:";
     const char* DATETIME_STRING = "Time:";
 }
 
@@ -94,7 +97,9 @@ MainWindow::MainWindow(const QString& marbleDataPath, const QVariantMap& cmdLine
         m_timeControlDialog( 0 ),
         m_downloadRegionDialog( 0 ),
         m_downloadRegionAction( 0 ),
+        m_kineticScrollingAction( 0 ),
         m_osmEditAction( 0 ),
+        m_zoomLabel( 0 ),
         m_mapViewWindow( 0 ),
         m_routingWindow( 0 ),
         m_trackingWindow( 0 ),
@@ -128,6 +133,11 @@ MainWindow::MainWindow(const QString& marbleDataPath, const QVariantMap& cmdLine
     // Load bookmark file. If it does not exist, a default one will be used.
     m_controlView->marbleModel()->bookmarkManager()->loadFile( "bookmarks/bookmarks.kml" );
 
+    const bool smallscreen = MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen;
+
+    if ( !smallscreen ) {
+        createToolBar();
+    }
     createActions();
     createMenus();
     createStatusBar();
@@ -140,6 +150,8 @@ MainWindow::MainWindow(const QString& marbleDataPath, const QVariantMap& cmdLine
 
     m_position = NOT_AVAILABLE;
     m_distance = marbleWidget()->distanceString();
+    m_zoom = QString::number( marbleWidget()->tileZoomLevel() );
+
     m_clock = QLocale().toString( m_controlView->marbleModel()->clockDateTime().addSecs( m_controlView->marbleModel()->clockTimezone() ), QLocale::ShortFormat );
     QMetaObject::invokeMethod(this,
                               "initObject", Qt::QueuedConnection,
@@ -240,10 +252,22 @@ void MainWindow::createActions()
      m_workOfflineAct->setCheckable( true );
      connect(m_workOfflineAct, SIGNAL(triggered( bool )), this, SLOT( workOffline( bool )));
 
+     m_kineticScrollingAction = new QAction( tr( "&Kinetic Scrolling" ), this);
+     m_kineticScrollingAction->setCheckable( true );
+     connect( m_kineticScrollingAction, SIGNAL( triggered( bool ) ), this, SLOT( toggleKineticScrolling( bool ) ) );
+
      m_showAtmosphereAct = new QAction( tr("&Atmosphere"), this);
+     m_showAtmosphereAct->setVisible( false );
      m_showAtmosphereAct->setCheckable( true );
      m_showAtmosphereAct->setStatusTip(tr("Show Atmosphere"));
      connect(m_showAtmosphereAct, SIGNAL(triggered( bool )), this, SLOT( showAtmosphere( bool )));
+     foreach ( RenderPlugin *plugin, m_controlView->marbleWidget()->renderPlugins() ) {
+         if ( plugin->nameId() == "atmosphere" ) {
+             m_showAtmosphereAct->setVisible( plugin->enabled() );
+             connect( plugin, SIGNAL( enabledChanged( bool ) ),
+                      m_showAtmosphereAct, SLOT( setVisible( bool ) ) );
+         }
+     }
 
      m_controlTimeAct = new QAction( tr( "&Time Control..." ), this );
      m_controlTimeAct->setStatusTip( tr( "Configure Time Control " ) );
@@ -282,6 +306,15 @@ void MainWindow::createActions()
      m_addBookmarkAct->setStatusTip(tr("Add Bookmark"));
      connect( m_addBookmarkAct, SIGNAL( triggered() ), this, SLOT( openEditBookmarkDialog() ) );
 
+     m_setHomeAct = new QAction( QIcon(":/icons/go-home.png"), tr( "&Set Home Location" ),this);
+     m_setHomeAct->setStatusTip( tr( "&Set Home Location" ) );
+     connect( m_setHomeAct, SIGNAL( triggered() ), this, SLOT( setHome() ) );
+
+     m_toggleBookmarkDisplayAct = new QAction(tr( "Show &Bookmarks" ), this);
+     m_toggleBookmarkDisplayAct->setStatusTip( tr( "Toggle display of Bookmarks" ) );
+     m_toggleBookmarkDisplayAct->setCheckable( true );
+     connect( m_toggleBookmarkDisplayAct, SIGNAL( triggered( bool ) ), this, SLOT( showBookmarks( bool ) ) );
+
      m_manageBookmarksAct = new QAction( QIcon( ":/icons/bookmarks-organize.png" ), tr( "&Manage Bookmarks" ), this);
      m_manageBookmarksAct->setStatusTip( tr( "Manage Bookmarks" ) );
      connect( m_manageBookmarksAct, SIGNAL( triggered() ), this, SLOT( manageBookmarks() ) );
@@ -297,6 +330,7 @@ void MainWindow::createMenus()
     // Do not create too many menu entries on a MID
     if( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
         menuBar()->addAction( m_workOfflineAct );
+        menuBar()->addAction( m_kineticScrollingAction );
         //menuBar()->addAction( m_sideBarAct );
         /** @todo: Full screen cannot be left on Maemo currently (shortcuts not working) */
         //menuBar()->addAction( m_fullScreenAct );
@@ -472,7 +506,7 @@ void MainWindow::createBookmarksListMenu( QMenu *m_bookmarksListMenu, const GeoD
             GeoDataCoordinates coordinates = (*i)->coordinate( m_controlView->marbleModel()->clockDateTime() );
             GeoDataLookAt coordinateToLookAt;
             coordinateToLookAt.setCoordinates( coordinates );
-            coordinateToLookAt.setRange( coordinates.altitude() );
+            coordinateToLookAt.setRange( marbleWidget()->lookAt().range() );
             var.setValue( coordinateToLookAt );
         } else {
             var.setValue( *lookAt );
@@ -487,6 +521,8 @@ void MainWindow::createBookmarkMenu()
 {
     m_bookmarkMenu->clear();
     m_bookmarkMenu->addAction( m_addBookmarkAct );
+    m_bookmarkMenu->addAction( m_toggleBookmarkDisplayAct );
+    m_bookmarkMenu->addAction( m_setHomeAct );
     m_bookmarkMenu->addAction( m_manageBookmarksAct );
 
     m_bookmarkMenu->addSeparator();
@@ -548,13 +584,23 @@ void MainWindow::manageBookmarks()
     delete dialog;
 }
 
+void MainWindow::setHome()
+{
+    MarbleWidget *widget = m_controlView->marbleWidget();
+    widget->model()->setHome( widget->centerLongitude(), widget->centerLatitude(), widget->zoom() );
+}
+
 void MainWindow::openEditBookmarkDialog()
 {
     MarbleWidget *widget = m_controlView->marbleWidget();
     QPointer<EditBookmarkDialog> dialog = new EditBookmarkDialog( widget->model()->bookmarkManager(), widget );
-    dialog->setLookAt( widget->lookAt() );
     dialog->setMarbleWidget( widget );
-    dialog->exec();
+    dialog->setCoordinates( widget->lookAt().coordinates() );
+    dialog->setRange( widget->lookAt().range() );
+    dialog->setReverseGeocodeName();
+    if ( dialog->exec() == QDialog::Accepted ) {
+        widget->model()->bookmarkManager()->addBookmark( dialog->folder(), dialog->bookmark() );
+    }
     delete dialog;
 }
 
@@ -657,6 +703,34 @@ void MainWindow::showFullScreen( bool isChecked )
     m_fullScreenAct->setChecked( isChecked ); // Sync state with the GUI
 }
 
+#ifdef Q_WS_MAEMO_5
+void MainWindow::setOrientation( Orientation orientation )
+{
+    switch ( orientation ) {
+    case OrientationAutorotate:
+       setAttribute( Qt::WA_Maemo5AutoOrientation );
+       break;
+    case OrientationLandscape:
+       setAttribute( Qt::WA_Maemo5LandscapeOrientation );
+       break;
+    case OrientationPortrait:
+       setAttribute( Qt::WA_Maemo5PortraitOrientation );
+       break;
+    }
+}
+
+MainWindow::Orientation MainWindow::orientation() const
+{
+    if ( testAttribute( Qt::WA_Maemo5LandscapeOrientation ) )
+        return OrientationLandscape;
+
+    if ( testAttribute( Qt::WA_Maemo5PortraitOrientation ) )
+        return OrientationPortrait;
+
+    return OrientationAutorotate;
+}
+#endif // Q_WS_MAEMO_5
+
 void MainWindow::showSideBar( bool isChecked )
 {
     m_controlView->setSideBarShown( isChecked );
@@ -704,6 +778,13 @@ void MainWindow::showClouds( bool isChecked )
     m_showCloudsAct->setChecked( isChecked ); // Sync state with the GUI
 }
 
+void MainWindow::showBookmarks( bool show )
+{
+    m_controlView->marbleModel()->bookmarkManager()->setShowBookmarks( show );
+
+    m_toggleBookmarkDisplayAct->setChecked( show ); // Sync state with the GUI
+}
+
 void MainWindow::workOffline( bool offline )
 {
     m_controlView->marbleControl()->setWorkOffline( offline );
@@ -711,6 +792,12 @@ void MainWindow::workOffline( bool offline )
     m_workOfflineAct->setChecked( offline ); // Sync state with the GUI
 }
 
+void MainWindow::toggleKineticScrolling( bool enabled )
+{
+    m_controlView->marbleWidget()->inputHandler()->setKineticScrollingEnabled( enabled );
+    m_kineticScrollingAction->setChecked( enabled ); // Sync state with the GUI
+}
+
 void MainWindow::showAtmosphere( bool isChecked )
 {
     m_controlView->marbleWidget()->setShowAtmosphere( isChecked );
@@ -805,6 +892,12 @@ void MainWindow::showDistance( const QString& distance )
     updateStatusBar();
 }
 
+void MainWindow::showZoom( int zoom )
+{
+    m_zoom = QString::number( zoom );
+    updateStatusBar();
+}
+
 void MainWindow::showDateTime()
 {
     m_clock = QLocale().toString( m_controlView->marbleModel()->clockDateTime().addSecs( m_controlView->marbleModel()->clockTimezone() ), QLocale::ShortFormat );
@@ -821,6 +914,10 @@ void MainWindow::updateStatusBar()
         m_distanceLabel->setText( QString( "%1 %2" )
         .arg( tr( DISTANCE_STRING ) ).arg( m_distance ) );
 
+    if ( m_zoomLabel )
+        m_zoomLabel->setText( QString( "%1 %2" )
+        .arg( tr( ZOOM_STRING ) ).arg( m_zoom ) );
+
     if ( m_clockLabel )
         m_clockLabel->setText( QString( "%1 %2" )
         .arg( tr( DATETIME_STRING ) ).arg( m_clock ) );
@@ -828,9 +925,28 @@ void MainWindow::updateStatusBar()
 
 void MainWindow::openFile()
 {
-    QStringList fileNames = QFileDialog::getOpenFileNames(this, tr("Open File"),
-                            m_lastFileOpenPath,
-                            tr("All Supported Files (*.gpx *.kml *.pnt *.osm);;GPS Data (*.gpx);;Google Earth KML (*.kml);;Micro World Database II (*.pnt);;OpenStreetMap Data (*.osm)"));
+    const PluginManager *const pluginManager = m_controlView->marbleModel()->pluginManager();
+
+    QStringList allFileExtensions;
+    QStringList filters;
+    foreach ( const ParseRunnerPlugin *plugin, pluginManager->parsingRunnerPlugins() ) {
+        if ( plugin->nameId() == "Cache" )
+            continue;
+
+        const QStringList fileExtensions = plugin->fileExtensions().replaceInStrings( QRegExp( "^" ), "*." );
+        const QString filter = QString( "%1 (%2)" ).arg( plugin->fileFormatDescription() ).arg( fileExtensions.join( " " ) );
+        filters << filter;
+        allFileExtensions << fileExtensions;
+    }
+
+    allFileExtensions.sort();  // sort since file extensions are visible under Windows
+    const QString allFileTypes = QString( "%1 (%2)" ).arg( tr( "All Supported Files" ) ).arg( allFileExtensions.join( " " ) );
+
+    filters.sort();
+    filters.prepend( allFileTypes );
+    const QString filter = filters.join( ";;" );
+
+    QStringList fileNames = QFileDialog::getOpenFileNames( this, tr( "Open File" ), m_lastFileOpenPath, filter );
 
     if ( !fileNames.isEmpty() ) {
         const QString firstFile = fileNames.first();
@@ -845,6 +961,14 @@ void MainWindow::openFile()
 void MainWindow::setupStatusBar()
 {
     statusBar()->setSizeGripEnabled( true );
+    statusBar()->setContextMenuPolicy( Qt::ActionsContextMenu );
+
+    QAction* toggleTileLevelAction = new QAction( "Show zoom level", statusBar() );
+    toggleTileLevelAction->setCheckable( true );
+    toggleTileLevelAction->setChecked( false );
+    connect( toggleTileLevelAction, SIGNAL( triggered( bool ) ),
+             this, SLOT( showZoomLevel( bool ) ) );
+    statusBar()->addAction( toggleTileLevelAction );
 
     m_positionLabel = new QLabel( );
     m_positionLabel->setIndent( 5 );
@@ -864,6 +988,15 @@ void MainWindow::setupStatusBar()
     m_distanceLabel->setFixedWidth( maxDistanceWidth );
     statusBar()->addPermanentWidget ( m_distanceLabel );
 
+    m_zoomLabel = new QLabel( );
+    m_zoomLabel->setIndent( 5 );
+    QString templateZoomString =
+        QString( "%1 00" ).arg(ZOOM_STRING);
+    int maxZoomWidth = fontMetrics().boundingRect(templateZoomString).width()
+                            + 2 * m_zoomLabel->margin() + 2 * m_zoomLabel->indent();
+    m_zoomLabel->setFixedWidth( maxZoomWidth );
+    // Not added here, but activated by the user with the context menu
+
     m_clockLabel = new QLabel( );
     m_clockLabel->setIndent( 5 );
     QString templateDateTimeString = QString( "%1 %2" ).arg( DATETIME_STRING , QLocale().toString( QDateTime::fromString ( "01:01:1000", "dd:mm:yyyy"), QLocale::ShortFormat ) );
@@ -876,6 +1009,8 @@ void MainWindow::setupStatusBar()
               this, SLOT( showPosition( QString ) ) );
     connect( marbleWidget(), SIGNAL( distanceChanged( QString ) ),
               this, SLOT( showDistance( QString ) ) );
+    connect( marbleWidget(), SIGNAL( tileLevelChanged( int ) ),
+            this, SLOT( showZoom( int ) ) );
     connect( m_controlView->marbleModel()->clock(), SIGNAL( timeChanged() ),
               this, SLOT( showDateTime() ) );
 
@@ -888,6 +1023,23 @@ void MainWindow::closeEvent(QCloseEvent *event)
     event->accept();
 }
 
+void MainWindow::createToolBar()
+{
+    QToolBar* toolBar = addToolBar( tr( "Main ToolBar" ) );
+    SearchInputWidget *searchField = new SearchInputWidget( this );
+    searchField->setCompletionModel( m_controlView->marbleModel()->placemarkModel() );
+    searchField->setMaximumWidth( 400 );
+    connect( searchField, SIGNAL( search( QString, SearchMode ) ), m_controlView, SLOT( search( QString, SearchMode ) ) );
+    connect( searchField, SIGNAL( centerOn( const GeoDataCoordinates &) ),
+             m_controlView->marbleWidget(), SLOT( centerOn( const GeoDataCoordinates & ) ) );
+    connect( m_controlView, SIGNAL( searchFinished() ), searchField, SLOT( disableSearchAnimation() ) );
+
+    QKeySequence searchShortcut( Qt::CTRL + Qt::Key_F );
+    searchField->setToolTip( QString( "Search for cities, addresses, points of interest and more (%1)" ).arg( searchShortcut.toString() ) );
+    new QShortcut( searchShortcut, searchField, SLOT( setFocus() ) );
+    toolBar->addWidget( searchField );
+}
+
 QString MainWindow::readMarbleDataPath()
 {
      QSettings settings("kde.org", "Marble Desktop Globe");
@@ -908,6 +1060,10 @@ void MainWindow::readSettings(const QVariantMap& overrideSettings)
          resize(settings.value("size", QSize(640, 480)).toSize());
          move(settings.value("pos", QPoint(200, 200)).toPoint());
          showFullScreen(settings.value("fullScreen", false ).toBool());
+#ifdef Q_WS_MAEMO_5
+         const Orientation orientation = (Orientation)settings.value( "orientation", (int)OrientationLandscape ).toInt();
+         setOrientation( orientation );
+#endif // Q_WS_MAEMO_5
          QByteArray sideBarState = settings.value( "sideBarState", QByteArray() ).toByteArray();
          m_controlView->setSideBarState( sideBarState );
          if( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
@@ -922,6 +1078,7 @@ void MainWindow::readSettings(const QVariantMap& overrideSettings)
          workOffline(settings.value("workOffline", false ).toBool());
          showAtmosphere(settings.value("showAtmosphere", true ).toBool());
          m_lastFileOpenPath = settings.value("lastFileOpenDir", QDir::homePath()).toString();
+         showBookmarks( settings.value( "showBookmarks", true ).toBool() );
      settings.endGroup();
 
      setUpdatesEnabled(false);
@@ -934,7 +1091,7 @@ void MainWindow::readSettings(const QVariantMap& overrideSettings)
          } else {
             mapThemeId = settings.value("mapTheme", m_controlView->defaultMapThemeId() ).toString();
          }
-         qDebug() << "ReadSettings: mapThemeId: " << mapThemeId;
+         mDebug() << Q_FUNC_INFO << "mapThemeId:" << mapThemeId;
          m_controlView->marbleWidget()->setMapThemeId( mapThemeId );
          bool const smallScreen = MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen;
          int const defaultProjection = smallScreen ? Mercator : Spherical;
@@ -964,7 +1121,18 @@ void MainWindow::readSettings(const QVariantMap& overrideSettings)
                     settings.value("quitLongitude", 0.0).toDouble(),
                     settings.value("quitLatitude", 0.0).toDouble() );
                 if (! isDistanceOverwritten) {
-                    m_controlView->marbleWidget()->zoomView( settings.value("quitZoom", 1000).toInt() );
+                    if ( settings.contains("quitRadius") ) {
+                        // provide a default value in case "quitRadius" contains garbage
+                        m_controlView->marbleWidget()->setRadius( settings.value("quitRadius", 1350).toInt() );
+                    }
+                    else if ( settings.contains("quitZoom") ) {
+                        // provide a default value in case "quitZoom" contains garbage
+                        m_controlView->marbleWidget()->zoomView( settings.value("quitZoom", 1000).toInt() );
+                    }
+                    else {
+                        // set radius to 1350 (Atlas theme's "sharp" radius) if Marble starts with a clean config
+                        m_controlView->marbleWidget()->setRadius( 1350 );
+                    }
                 }
                 break;
             case Marble::ShowHomeLocation:
@@ -981,6 +1149,8 @@ void MainWindow::readSettings(const QVariantMap& overrideSettings)
          bool isLocked = settings.value( "lockFloatItemPositions", false ).toBool();
          m_lockFloatItemsAct->setChecked( isLocked );
          lockPosition(isLocked);
+
+         toggleKineticScrolling( settings.value( "kineticScrolling", !smallScreen ).toBool() );
      settings.endGroup();
 
      settings.beginGroup( "Sun" );
@@ -1012,6 +1182,19 @@ void MainWindow::readSettings(const QVariantMap& overrideSettings)
          routingManager->setShowGuidanceModeStartupWarning( startupWarning );
          routingManager->setLastOpenPath( settings.value( "lastRouteOpenPath", QDir::homePath() ).toString() );
          routingManager->setLastSavePath( settings.value( "lastRouteSavePath", QDir::homePath() ).toString() );
+
+         QColor tempColor;
+         tempColor = QColor( settings.value( "routeColorStandard", oxygenSkyBlue4.name() ).toString() );
+         tempColor.setAlpha( settings.value( "routeAlphaStandard", 200 ).toInt() );
+         routingManager->setRouteColorStandard( tempColor );
+
+         tempColor = QColor( settings.value( "routeColorHighlighted", oxygenSeaBlue2.name() ).toString() );
+         tempColor.setAlpha( settings.value( "routeAlphaHighlighted", 200 ).toInt() );
+         routingManager->setRouteColorHighlighted( tempColor );
+
+         tempColor = QColor( settings.value( "routeColorAlternative", oxygenAluminumGray4.name() ).toString() );
+         tempColor.setAlpha( settings.value( "routeAlphaAlternative", 200 ).toInt() );
+         routingManager->setRouteColorAlternative( tempColor );
      }
      settings.endGroup();
 
@@ -1050,20 +1233,19 @@ void MainWindow::readSettings(const QVariantMap& overrideSettings)
     settings.endGroup();
 
     settings.beginGroup( "Plugins");
+    PositionTracking* tracking = m_controlView->marbleModel()->positionTracking();
+    tracking->readSettings();
     QString positionProvider = settings.value( "activePositionTrackingPlugin", QString() ).toString();
     if ( !positionProvider.isEmpty() ) {
-        PositionTracking* tracking = m_controlView->marbleModel()->positionTracking();
         const PluginManager* pluginManager = m_controlView->marbleModel()->pluginManager();
-        QList<PositionProviderPlugin*> plugins = pluginManager->createPositionProviderPlugins();
-        foreach( PositionProviderPlugin* plugin, plugins ) {
+        foreach( const PositionProviderPlugin* plugin, pluginManager->positionProviderPlugins() ) {
             if ( plugin->nameId() == positionProvider ) {
-                plugins.removeOne( plugin );
-                plugin->setMarbleModel( m_controlView->marbleModel() );
-                tracking->setPositionProviderPlugin( plugin );
+                PositionProviderPlugin* instance = plugin->newInstance();
+                instance->setMarbleModel( m_controlView->marbleModel() );
+                tracking->setPositionProviderPlugin( instance );
                 break;
             }
         }
-        qDeleteAll( plugins );
     }
     settings.endGroup();
 
@@ -1101,6 +1283,9 @@ void MainWindow::writeSettings()
          settings.setValue( "size", size() );
          settings.setValue( "pos", pos() );
          settings.setValue( "fullScreen", m_fullScreenAct->isChecked() );
+#ifdef Q_WS_MAEMO_5
+         settings.setValue( "orientation", (int)orientation() );
+#endif // Q_WS_MAEMO_5
          settings.setValue( "sideBar", m_sideBarAct->isChecked() );
          settings.setValue( "sideBarState", m_controlView->sideBarState() );
          settings.setValue( "statusBar", m_statusBarAct->isChecked() );
@@ -1108,6 +1293,7 @@ void MainWindow::writeSettings()
          settings.setValue( "workOffline", m_workOfflineAct->isChecked() );
          settings.setValue( "showAtmosphere", m_showAtmosphereAct->isChecked() );
          settings.setValue( "lastFileOpenDir", m_lastFileOpenPath );
+         settings.setValue( "showBookmarks", m_toggleBookmarkDisplayAct->isChecked() );
      settings.endGroup();
 
      settings.beginGroup( "MarbleWidget" );
@@ -1129,13 +1315,14 @@ void MainWindow::writeSettings()
          // Get the 'quit' values from the widget and store them in the settings.
          qreal  quitLon = m_controlView->marbleWidget()->centerLongitude();
          qreal  quitLat = m_controlView->marbleWidget()->centerLatitude();
-         int    quitZoom = m_controlView->marbleWidget()->zoom();
+         const int quitRadius = m_controlView->marbleWidget()->radius();
 
          settings.setValue( "quitLongitude", quitLon );
          settings.setValue( "quitLatitude", quitLat );
-         settings.setValue( "quitZoom", quitZoom );
+         settings.setValue( "quitRadius", quitRadius );
 
          settings.setValue( "lockFloatItemPositions", m_lockFloatItemsAct->isChecked() );
+         settings.setValue( "kineticScrolling", m_controlView->marbleWidget()->inputHandler()->kineticScrollingEnabled() );
      settings.endGroup();
 
      settings.beginGroup( "Sun" );
@@ -1180,6 +1367,7 @@ void MainWindow::writeSettings()
      settings.beginGroup( "Plugins");
      QString positionProvider;
      PositionTracking* tracking = m_controlView->marbleModel()->positionTracking();
+     tracking->writeSettings();
      if ( tracking && tracking->positionProviderPlugin() ) {
          positionProvider = tracking->positionProviderPlugin()->nameId();
      }
@@ -1215,6 +1403,12 @@ void MainWindow::writeSettings()
          settings.setValue( "showGuidanceModeStartupWarning", routingManager->showGuidanceModeStartupWarning() );
          settings.setValue( "lastRouteOpenPath", routingManager->lastOpenPath() );
          settings.setValue( "lastRouteSavePath", routingManager->lastSavePath() );
+         settings.setValue( "routeColorStandard", routingManager->routeColorStandard().name() );
+         settings.setValue( "routeAlphaStandard", routingManager->routeColorStandard().alpha() );
+         settings.setValue( "routeColorHighlighted", routingManager->routeColorHighlighted().name() );
+         settings.setValue( "routeAlphaHighlighted", routingManager->routeColorHighlighted().alpha() );
+         settings.setValue( "routeColorAlternative", routingManager->routeColorAlternative().name() );
+         settings.setValue( "routeAlphaAlternative", routingManager->routeColorAlternative().alpha() );
      }
      settings.endGroup();
 
@@ -1233,7 +1427,7 @@ void MainWindow::editSettings()
 
 void MainWindow::updateSettings()
 {
-    qDebug() << "Updating Settings ...";
+    mDebug() << Q_FUNC_INFO << "Updating Settings ...";
 
     // FIXME: Font doesn't get updated instantly.
     m_controlView->marbleWidget()->setDefaultFont( m_configDialog->mapFont() );
@@ -1277,7 +1471,7 @@ void MainWindow::showDownloadRegionDialog()
                  SLOT( disconnectDownloadRegionDialog() ));
     }
     // FIXME: get allowed range from current map theme
-    m_downloadRegionDialog->setAllowedTileLevelRange( 0, 18 );
+    m_downloadRegionDialog->setAllowedTileLevelRange( 0, 16 );
     m_downloadRegionDialog->setSelectionMethod( DownloadRegionDialog::VisibleRegionMethod );
     ViewportParams const * const viewport =
         m_controlView->marbleWidget()->viewport();
@@ -1310,11 +1504,10 @@ void MainWindow::disconnectDownloadRegionDialog()
 void MainWindow::downloadRegion()
 {
     Q_ASSERT( m_downloadRegionDialog );
-    QString const mapThemeId = m_controlView->marbleWidget()->mapThemeId();
-    QString const sourceDir = mapThemeId.left( mapThemeId.lastIndexOf( '/' ));
-    mDebug() << "downloadRegion mapThemeId:" << mapThemeId << sourceDir;
     QVector<TileCoordsPyramid> const pyramid = m_downloadRegionDialog->region();
-    m_controlView->marbleWidget()->downloadRegion( sourceDir, pyramid );
+    if ( !pyramid.isEmpty() ) {
+        m_controlView->marbleWidget()->downloadRegion( pyramid );
+    }
 }
 
 void MainWindow::printMapScreenShot()
@@ -1330,13 +1523,15 @@ void MainWindow::printMapScreenShot()
 void MainWindow::showMapViewDialog()
 {
     if( !m_mapViewWindow ) {
-        m_mapViewWindow = new StackableWindow( this );
+        m_mapViewWindow = new QDialog( this );
         m_mapViewWindow->setWindowTitle( tr( "Map View - Marble" ) );
 
+        QVBoxLayout *layout = new QVBoxLayout;
+        m_mapViewWindow->setLayout( layout );
+
         MapViewWidget *mapViewWidget = new MapViewWidget( m_mapViewWindow );
         mapViewWidget->setMarbleWidget( m_controlView->marbleWidget() );
-
-        m_mapViewWindow->setCentralWidget( mapViewWidget );
+        layout->addWidget( mapViewWidget );
     }
 
     m_mapViewWindow->show();
@@ -1366,6 +1561,7 @@ void MainWindow::showRoutingDialog()
 
         QScrollArea* scrollArea = new QScrollArea;
         m_routingWidget->setMinimumWidth( 760 );
+        scrollArea->setWidgetResizable( true );
         scrollArea->setWidget( m_routingWidget );
 
         QAction *openAction = new QAction( tr( "Open Route..." ), this );
@@ -1376,16 +1572,6 @@ void MainWindow::showRoutingDialog()
         connect( saveAction, SIGNAL( triggered() ), m_routingWidget, SLOT( saveRoute() ) );
         m_routingWindow->menuBar()->addAction( saveAction );
 
-        QAction* reverseAction = new QAction( tr( "Reverse Route" ), this );
-        RoutingManager * const manager = m_controlView->marbleModel()->routingManager();
-        connect( reverseAction, SIGNAL( triggered() ), manager, SLOT( reverseRoute() ) );
-        m_routingWindow->menuBar()->addAction( reverseAction );
-
-        /** @todo: Change 'Clear' to 'Clear Route' after string freeze */
-        QAction* clearAction = new QAction( tr( "Clear" ), this );
-        connect( clearAction, SIGNAL( triggered() ), manager, SLOT( clearRoute() ) );
-        m_routingWindow->menuBar()->addAction( clearAction );
-
         m_routingWindow->setCentralWidget( scrollArea );
     }
 
@@ -1477,4 +1663,14 @@ void MainWindow::showGoToDialog()
     }
 }
 
+void MainWindow::showZoomLevel(bool show)
+{
+    if ( show ) {
+        m_zoomLabel->show();
+        statusBar()->insertPermanentWidget( 2, m_zoomLabel );
+    } else {
+        statusBar()->removeWidget( m_zoomLabel );
+    }
+}
+
 #include "QtMainWindow.moc"
