@@ -9,7 +9,7 @@
 // Copyright 2007      Inge Wallin  <ingwa@kde.org>
 // Copyright 2008      Carlos Licea <carlos.licea@kdemail.net>
 // Copyright 2009      Jens-Michael Hoffmann <jensmh@gmx.de>
-// Copyright 2010-2011 Bernhard Beschow <bbeschow@cs.tu-berlin.de>
+// Copyright 2010-2012 Bernhard Beschow <bbeschow@cs.tu-berlin.de>
 //
 
 
@@ -32,18 +32,16 @@
 #endif
 
 // Marble
-#include "layers/AtmosphereLayer.h"
 #include "layers/FogLayer.h"
 #include "layers/FpsLayer.h"
 #include "layers/GeometryLayer.h"
+#include "layers/GroundLayer.h"
 #include "layers/MarbleSplashLayer.h"
-#include "layers/MeasureTool.h"
-#include "layers/PlacemarkLayout.h"
+#include "layers/PlacemarkLayer.h"
 #include "layers/TextureLayer.h"
 #include "layers/VectorMapBaseLayer.h"
 #include "layers/VectorMapLayer.h"
 #include "AbstractFloatItem.h"
-#include "AbstractProjection.h"
 #include "GeoDataTreeModel.h"
 #include "GeoPainter.h"
 #include "GeoSceneDocument.h"
@@ -61,7 +59,6 @@
 #include "MarbleModel.h"
 #include "RenderPlugin.h"
 #include "SunLocator.h"
-#include "TextureColorizer.h"
 #include "TileCoordsPyramid.h"
 #include "TileCreator.h"
 #include "TileCreatorDialog.h"
@@ -126,19 +123,17 @@ class MarbleMapPrivate
     bool             m_showFrameRate;
 
     VectorComposer   m_veccomposer;
-    TextureColorizer *m_texcolorizer;
 
     LayerManager     m_layerManager;
     MarbleSplashLayer m_marbleSplashLayer;
     MarbleMap::CustomPaintLayer m_customPaintLayer;
     GeometryLayer            m_geometryLayer;
-    AtmosphereLayer          m_atmosphereLayer;
     FogLayer                 m_fogLayer;
+    GroundLayer              m_groundLayer;
     VectorMapBaseLayer       m_vectorMapBaseLayer;
     VectorMapLayer   m_vectorMapLayer;
     TextureLayer     m_textureLayer;
-    PlacemarkLayout  m_placemarkLayout;
-    MeasureTool      m_measureTool;
+    PlacemarkLayer   m_placemarkLayer;
 };
 
 MarbleMapPrivate::MarbleMapPrivate( MarbleMap *parent, MarbleModel *model )
@@ -147,47 +142,27 @@ MarbleMapPrivate::MarbleMapPrivate( MarbleMap *parent, MarbleModel *model )
           m_viewParams(),
           m_showFrameRate( false ),
           m_veccomposer(),
-          m_texcolorizer( 0 ),
           m_layerManager( model, parent ),
           m_customPaintLayer( parent ),
           m_geometryLayer( model->treeModel() ),
           m_vectorMapBaseLayer( &m_veccomposer ),
           m_vectorMapLayer( &m_veccomposer ),
-          m_textureLayer( model->downloadManager(), model->sunLocator() ),
-          m_placemarkLayout( model->placemarkModel(), model->placemarkSelectionModel(), model->clock(), parent ),
-          m_measureTool( model )
+          m_textureLayer( model->downloadManager(), model->sunLocator(), &m_veccomposer ),
+          m_placemarkLayer( model->placemarkModel(), model->placemarkSelectionModel(), model->clock() )
 {
     m_layerManager.addLayer( &m_fogLayer );
-    m_layerManager.addLayer( &m_measureTool );
+    m_layerManager.addLayer( &m_groundLayer );
     m_layerManager.addLayer( &m_geometryLayer );
-    m_layerManager.addLayer( &m_placemarkLayout );
+    m_layerManager.addLayer( &m_placemarkLayer );
     m_layerManager.addLayer( &m_customPaintLayer );
 
-    QList<RenderPlugin *> pluginList = m_layerManager.renderPlugins();
-    QList<RenderPlugin *>::const_iterator i = pluginList.constBegin();
-    QList<RenderPlugin *>::const_iterator const end = pluginList.constEnd();
-    for (; i != end; ++i ) {
-        if ( (*i)->nameId() == "sun" ) {
-            (*i)->setVisible( false );
-        }
-    }
-
     QObject::connect( m_model, SIGNAL( themeChanged( QString ) ),
                       parent, SLOT( updateMapTheme() ) );
 
     QObject::connect( &m_veccomposer, SIGNAL( datasetLoaded() ),
                       parent, SIGNAL( repaintNeeded() ));
 
-    QObject::connect( model->treeModel(), SIGNAL( dataChanged( QModelIndex, QModelIndex ) ),
-                      &m_placemarkLayout, SLOT( setCacheData() ) );
-    QObject::connect( model->treeModel(), SIGNAL( layoutChanged() ),
-                      &m_placemarkLayout, SLOT( setCacheData() ) );
-    QObject::connect( model->treeModel(), SIGNAL( modelReset() ),
-                      &m_placemarkLayout, SLOT( setCacheData() ) );
-    QObject::connect( model->treeModel(), SIGNAL( treeChanged() ),
-                      &m_placemarkLayout, SLOT( setCacheData() ) );
-
-    QObject::connect( &m_placemarkLayout, SIGNAL( repaintNeeded()),
+    QObject::connect( &m_placemarkLayer, SIGNAL( repaintNeeded()),
                       parent, SIGNAL( repaintNeeded() ));
 
     QObject::connect ( &m_layerManager, SIGNAL( pluginSettingsChanged() ),
@@ -196,15 +171,8 @@ MarbleMapPrivate::MarbleMapPrivate( MarbleMap *parent, MarbleModel *model )
                        parent,        SIGNAL( repaintNeeded( QRegion ) ) );
     QObject::connect ( &m_layerManager, SIGNAL( renderPluginInitialized( RenderPlugin * ) ),
                        parent,        SIGNAL( renderPluginInitialized( RenderPlugin * ) ) );
-
-    QObject::connect( model->treeModel(), SIGNAL( dataChanged( QModelIndex, QModelIndex ) ),
-                      &m_geometryLayer, SLOT( invalidateScene() ) );
-    QObject::connect( model->treeModel(), SIGNAL( layoutChanged() ),
-                      &m_geometryLayer, SLOT( invalidateScene() ) );
-    QObject::connect( model->treeModel(), SIGNAL( modelReset() ),
-                      &m_geometryLayer, SLOT( invalidateScene() ) );
-    QObject::connect( model->treeModel(), SIGNAL( treeChanged() ),
-                      &m_geometryLayer, SLOT( invalidateScene() ) );
+    QObject::connect ( &m_layerManager, SIGNAL( visibilityChanged( const QString &, bool ) ),
+                       parent,        SLOT( setPropertyValue( const QString &, bool ) ) );
 
     QObject::connect( &m_geometryLayer, SIGNAL( repaintNeeded()),
                       parent, SIGNAL( repaintNeeded() ));
@@ -213,28 +181,31 @@ MarbleMapPrivate::MarbleMapPrivate( MarbleMap *parent, MarbleModel *model )
                       parent, SIGNAL( tileLevelChanged( int ) ) );
     QObject::connect( &m_textureLayer, SIGNAL( repaintNeeded() ),
                       parent, SIGNAL( repaintNeeded() ) );
+
+    QObject::connect( parent, SIGNAL( visibleLatLonAltBoxChanged( const GeoDataLatLonAltBox & ) ),
+                      parent, SIGNAL( repaintNeeded() ) );
 }
 
 void MarbleMapPrivate::updateProperty( const QString &name, bool show )
 {
     // earth
     if ( name == "places" ) {
-        m_placemarkLayout.setShowPlaces( show );
+        m_placemarkLayer.setShowPlaces( show );
     } else if ( name == "cities" ) {
-        m_placemarkLayout.setShowCities( show );
+        m_placemarkLayer.setShowCities( show );
     } else if ( name == "terrain" ) {
-        m_placemarkLayout.setShowTerrain( show );
+        m_placemarkLayer.setShowTerrain( show );
     } else if ( name == "otherplaces" ) {
-        m_placemarkLayout.setShowOtherPlaces( show );
+        m_placemarkLayer.setShowOtherPlaces( show );
     }
 
     // other planets
     else if ( name == "landingsites" ) {
-        m_placemarkLayout.setShowLandingSites( show );
+        m_placemarkLayer.setShowLandingSites( show );
     } else if ( name == "craters" ) {
-        m_placemarkLayout.setShowCraters( show );
+        m_placemarkLayer.setShowCraters( show );
     } else if ( name == "maria" ) {
-        m_placemarkLayout.setShowMaria( show );
+        m_placemarkLayer.setShowMaria( show );
     }
 
     else if ( name == "waterbodies" ) {
@@ -252,10 +223,10 @@ void MarbleMapPrivate::updateProperty( const QString &name, bool show )
     }
 
     else if ( name == "relief" ) {
-        if ( m_texcolorizer ) {
-            m_texcolorizer->setShowRelief( show );
-        }
+       m_textureLayer.setShowRelief( show );
     }
+
+    m_layerManager.setVisible( name, show );
 }
 
 // ----------------------------------------------------------------
@@ -291,11 +262,10 @@ MarbleMap::~MarbleMap()
 
     d->m_layerManager.removeLayer( &d->m_customPaintLayer );
     d->m_layerManager.removeLayer( &d->m_geometryLayer );
-    d->m_layerManager.removeLayer( &d->m_measureTool );
     d->m_layerManager.removeLayer( &d->m_fogLayer );
-    d->m_layerManager.removeLayer( &d->m_placemarkLayout );
+    d->m_layerManager.removeLayer( &d->m_placemarkLayer );
     d->m_layerManager.removeLayer( &d->m_textureLayer );
-    d->m_layerManager.removeLayer( &d->m_atmosphereLayer );
+    d->m_layerManager.removeLayer( &d->m_groundLayer );
     d->m_layerManager.removeLayer( &d->m_vectorMapLayer );
     d->m_layerManager.removeLayer( &d->m_vectorMapBaseLayer );
     delete d;
@@ -339,10 +309,16 @@ MapQuality MarbleMap::mapQuality() const
 
 void MarbleMap::setViewContext( ViewContext viewContext )
 {
+    const MapQuality oldQuality = d->m_viewParams.mapQuality();
+
     d->m_viewParams.setViewContext( viewContext );
 
-    // Update texture map during the repaint that follows:
-    d->m_textureLayer.setNeedsUpdate();
+    if ( d->m_viewParams.mapQuality() != oldQuality ) {
+        // Update texture map during the repaint that follows:
+        d->m_textureLayer.setNeedsUpdate();
+
+        emit repaintNeeded();
+    }
 }
 
 ViewContext MarbleMap::viewContext() const
@@ -386,15 +362,22 @@ int MarbleMap::radius() const
 
 void MarbleMap::setRadius( int radius )
 {
+    const int oldRadius = d->m_viewport.radius();
+
     d->m_viewport.setRadius( radius );
 
     d->m_textureLayer.setNeedsUpdate();
+
+    if ( oldRadius != d->m_viewport.radius() ) {
+        emit radiusChanged( radius );
+        emit visibleLatLonAltBoxChanged( d->m_viewport.viewLatLonAltBox() );
+    }
 }
 
 
 int MarbleMap::preferredRadiusCeil( int radius )
 {
-    if ( d->m_textureLayer.tileZoomLevel() < 0 )
+    if ( !d->m_layerManager.internalLayers().contains( &d->m_textureLayer ) )
         return radius;
 
     return d->m_textureLayer.preferredRadiusCeil( radius );
@@ -403,7 +386,7 @@ int MarbleMap::preferredRadiusCeil( int radius )
 
 int MarbleMap::preferredRadiusFloor( int radius )
 {
-    if ( d->m_textureLayer.tileZoomLevel() < 0 )
+    if ( !d->m_layerManager.internalLayers().contains( &d->m_textureLayer ) )
         return radius;
 
     return d->m_textureLayer.preferredRadiusFloor( radius );
@@ -450,7 +433,7 @@ int  MarbleMap::maximumZoom() const
 
 QVector<const GeoDataPlacemark*> MarbleMap::whichFeatureAt( const QPoint& curpos ) const
 {
-    return d->m_placemarkLayout.whichPlacemarkAt( curpos );
+    return d->m_placemarkLayer.whichPlacemarkAt( curpos );
 }
 
 void MarbleMap::reload() const
@@ -458,7 +441,7 @@ void MarbleMap::reload() const
     d->m_textureLayer.reload();
 }
 
-void MarbleMap::downloadRegion( const QString& sourceDir, QVector<TileCoordsPyramid> const & pyramid )
+void MarbleMap::downloadRegion( QVector<TileCoordsPyramid> const & pyramid )
 {
     Q_ASSERT( textureLayer() );
     Q_ASSERT( !pyramid.isEmpty() );
@@ -484,8 +467,8 @@ void MarbleMap::downloadRegion( const QString& sourceDir, QVector<TileCoordsPyra
             coords.getCoords( &x1, &y1, &x2, &y2 );
             for ( int x = x1; x <= x2; ++x ) {
                 for ( int y = y1; y <= y2; ++y ) {
-                    TileId const tileId( sourceDir, level, x, y );
-                    tileIdSet.insert( tileId );
+                    TileId const stackedTileId( 0, level, x, y );
+                    tileIdSet.insert( stackedTileId );
                     // FIXME: use lazy evaluation to not generate up to 100k tiles in one go
                     // this can take considerable time even on very fast systems
                     // in contrast generating the TileIds on the fly when they are needed
@@ -495,7 +478,8 @@ void MarbleMap::downloadRegion( const QString& sourceDir, QVector<TileCoordsPyra
          }
          QSetIterator<TileId> i( tileIdSet );
          while( i.hasNext() ) {
-              textureLayer()->downloadTile( i.next() );
+             TileId const stackedTileId = i.next();
+             d->m_textureLayer.downloadStackedTile( stackedTileId );
          }
          tilesCount += tileIdSet.count();
      }
@@ -696,17 +680,14 @@ void MarbleMap::setProjection( Projection projection )
 bool MarbleMap::screenCoordinates( qreal lon, qreal lat,
                                    qreal& x, qreal& y ) const
 {
-    return d->m_viewport.currentProjection()
-        ->screenCoordinates( lon * DEG2RAD, lat * DEG2RAD,
-                             &d->m_viewport, x, y );
+    return d->m_viewport.screenCoordinates( lon * DEG2RAD, lat * DEG2RAD, x, y );
 }
 
 bool MarbleMap::geoCoordinates( int x, int y,
                                 qreal& lon, qreal& lat,
                                 GeoDataCoordinates::Unit unit ) const
 {
-    return d->m_viewport.currentProjection()
-        ->geoCoordinates( x, y, &d->m_viewport, lon, lat, unit );
+    return d->m_viewport.geoCoordinates( x, y, lon, lat, unit );
 }
 
 // Used to be paintEvent()
@@ -726,8 +707,8 @@ void MarbleMap::paint( GeoPainter &painter, const QRect &dirtyRect )
     d->m_layerManager.renderLayers( &painter, &d->m_viewport );
 
     if ( d->m_showFrameRate ) {
-        FpsLayer fpsLayer( &t );
-        fpsLayer.render( &painter, &d->m_viewport );
+        FpsLayer fpsPainter( &t );
+        fpsPainter.paint( &painter );
     }
 
     const qreal fps = 1000.0 / (qreal)( t.elapsed() );
@@ -754,20 +735,18 @@ void MarbleMapPrivate::updateMapTheme()
     m_layerManager.removeLayer( &m_textureLayer );
     m_layerManager.removeLayer( &m_vectorMapLayer );
     m_layerManager.removeLayer( &m_vectorMapBaseLayer );
-
-    m_textureLayer.setTextureColorizer( 0 );
-    delete m_texcolorizer;
-    m_texcolorizer = 0;
+    m_layerManager.removeLayer( &m_groundLayer );
 
     QObject::connect( m_model->mapTheme()->settings(), SIGNAL( valueChanged( const QString &, bool ) ),
                       q, SLOT( updateProperty( const QString &, bool ) ) );
 
     q->setPropertyValue( "clouds_data", m_viewParams.showClouds() );
 
-    // NOTE due to frequent regressions: 
-    // Do NOT take it for granted that there is any TEXTURE or VECTOR data AVAILABLE
-    // at this point. Some themes do NOT have either vector or texture data!
-    
+    if ( !m_model->mapTheme()->map()->hasTextureLayers() ) {
+        m_groundLayer.setColor( m_model->mapTheme()->map()->backgroundColor() );
+        m_layerManager.addLayer( &m_groundLayer );
+    }
+
     // Check whether there is a vector layer available:
     if ( m_model->mapTheme()->map()->hasVectorLayers() ) {
         m_veccomposer.setShowWaterBodies( q->propertyValue( "waterbodies" ) );
@@ -783,27 +762,27 @@ void MarbleMapPrivate::updateMapTheme()
         // Just as with textures, this is a workaround for DGML2 to
         // emulate the old behaviour.
 
-        GeoSceneLayer *layer = m_model->mapTheme()->map()->layer( "mwdbii" );
+        const GeoSceneLayer *layer = m_model->mapTheme()->map()->layer( "mwdbii" );
         if ( layer ) {
-            GeoSceneVector *vector = 0;
+            const GeoSceneVector *vector = 0;
 
-            vector = static_cast<GeoSceneVector*>( layer->dataset("pdiffborder") );
+            vector = static_cast<const GeoSceneVector*>( layer->dataset("pdiffborder") );
             if ( vector )
                 m_veccomposer.setCountryBorderColor( vector->pen().color() );
 
-            vector = static_cast<GeoSceneVector*>( layer->dataset("rivers") );
+            vector = static_cast<const GeoSceneVector*>( layer->dataset("rivers") );
             if ( vector )
                 m_veccomposer.setRiverColor( vector->pen().color() );
 
-            vector = static_cast<GeoSceneVector*>( layer->dataset("pusa48") );
+            vector = static_cast<const GeoSceneVector*>( layer->dataset("pusa48") );
             if ( vector )
                 m_veccomposer.setStateBorderColor( vector->pen().color() );
 
-            vector = static_cast<GeoSceneVector*>( layer->dataset("plake") );
+            vector = static_cast<const GeoSceneVector*>( layer->dataset("plake") );
             if ( vector )
                 m_veccomposer.setLakeColor( vector->pen().color() );
 
-            vector = static_cast<GeoSceneVector*>( layer->dataset("pcoast") );
+            vector = static_cast<const GeoSceneVector*>( layer->dataset("pcoast") );
             if ( vector )
             {
                 m_veccomposer.setLandColor( vector->brush().color() );
@@ -820,23 +799,20 @@ void MarbleMapPrivate::updateMapTheme()
         m_layerManager.addLayer( &m_vectorMapLayer );
     }
 
-    // NOTE due to frequent regressions: 
-    // Do NOT take it for granted that there is any TEXTURE or VECTOR data AVAILABLE
-    // at this point.
-
     // Check whether there is a texture layer available:
     if ( m_model->mapTheme()->map()->hasTextureLayers() ) {
-        GeoSceneSettings *const settings = m_model->mapTheme()->settings();
-        GeoSceneGroup *const textureLayerSettings = settings ? settings->group( "Texture Layers" ) : 0;
+        const GeoSceneSettings *const settings = m_model->mapTheme()->settings();
+        const GeoSceneGroup *const textureLayerSettings = settings ? settings->group( "Texture Layers" ) : 0;
 
         const GeoSceneHead *const head = m_model->mapTheme()->head();
         const GeoSceneMap *const map = m_model->mapTheme()->map();
         const GeoSceneLayer *const sceneLayer = ( head && map ) ? map->layer( head->theme() ) : 0;
 
-        QVector<const GeoSceneTexture *> textures;
+        bool textureLayersOk = true;
+        QVector<const GeoSceneTiled *> textures;
         if ( sceneLayer ) {
             foreach ( const GeoSceneAbstractDataset *pos, sceneLayer->datasets() ) {
-                const GeoSceneTexture *const texture = dynamic_cast<GeoSceneTexture const *>( pos );
+                const GeoSceneTiled *const texture = dynamic_cast<GeoSceneTiled const *>( pos );
                 if ( !texture )
                     continue;
 
@@ -870,26 +846,21 @@ void MarbleMapPrivate::updateMapTheme()
                     delete tileCreatorDlg;
                 }
 
-                if ( TileLoader::baseTilesAvailable( *texture ) ) {
-                    textures.append( texture );
-                } else {
-                    qWarning() << "Base tiles for" << sourceDir << "not available. Skipping.";
+                textures.append( texture );
+                if ( !TileLoader::baseTilesAvailable( *texture ) ) {
+                    qWarning() << "Base tiles for" << sourceDir << "not available.";
                 }
             }
         }
 
-        m_textureLayer.setMapTheme( textures, textureLayerSettings );
-
-        m_textureLayer.setupTextureMapper( m_viewport.projection() );
-
+        QString seafile, landfile;
         if( !m_model->mapTheme()->map()->filters().isEmpty() ) {
-            GeoSceneFilter *filter= m_model->mapTheme()->map()->filters().first();
+            const GeoSceneFilter *filter= m_model->mapTheme()->map()->filters().first();
 
             if( filter->type() == "colorize" ) {
                  //no need to look up with MarbleDirs twice so they are left null for now
-                QString seafile, landfile;
-                QList<GeoScenePalette*> palette = filter->palette();
-                foreach ( GeoScenePalette *curPalette, palette ) {
+                QList<const GeoScenePalette*> palette = filter->palette();
+                foreach ( const GeoScenePalette *curPalette, palette ) {
                     if( curPalette->type() == "sea" ) {
                         seafile = MarbleDirs::path( curPalette->file() );
                     } else if( curPalette->type() == "land" ) {
@@ -901,39 +872,45 @@ void MarbleMapPrivate::updateMapTheme()
                     seafile = MarbleDirs::path( "seacolors.leg" );
                 if( landfile.isEmpty() )
                     landfile = MarbleDirs::path( "landcolors.leg" );
-
-                m_texcolorizer = new TextureColorizer( seafile, landfile, &m_veccomposer, q );
-                m_texcolorizer->setShowRelief( q->showRelief() );
-
-                m_textureLayer.setTextureColorizer( m_texcolorizer );
             }
         }
 
-        m_layerManager.addLayer( &m_textureLayer );
-    }
+        m_textureLayer.setMapTheme( textures, textureLayerSettings, seafile, landfile );
+        m_textureLayer.setupTextureMapper( m_viewport.projection() );
+        m_textureLayer.setShowRelief( q->showRelief() );
 
-    // NOTE due to frequent regressions: 
-    // Do NOT take it for granted that there is any TEXTURE or VECTOR data AVAILABLE
-    // at this point!
+        if ( textureLayersOk ) {
+            m_layerManager.addLayer( &m_textureLayer );
+        }
+    }
+    else {
+        m_textureLayer.setMapTheme( QVector<const GeoSceneTiled *>(), 0, "", "" );
+    }
 
     // earth
-    bool value;
-    if ( m_model->mapTheme()->settings()->propertyValue( "places", value ) ) {
-        m_placemarkLayout.setShowPlaces( value );
+    m_placemarkLayer.setShowPlaces( q->showPlaces() );
+
+    m_placemarkLayer.setShowCities( q->showCities() );
+    m_placemarkLayer.setShowTerrain( q->showTerrain() );
+    m_placemarkLayer.setShowOtherPlaces( q->showOtherPlaces() );
+    m_placemarkLayer.setShowLandingSites( q->propertyValue("landingsites") );
+    m_placemarkLayer.setShowCraters( q->propertyValue("craters") );
+    m_placemarkLayer.setShowMaria( q->propertyValue("maria") );
+
+    GeoDataFeature::setDefaultLabelColor( m_model->mapTheme()->map()->labelColor() );
+    m_placemarkLayer.requestStyleReset();
+
+    foreach( RenderPlugin *renderPlugin, m_layerManager.renderPlugins() ) {
+        bool propertyAvailable = false;
+        m_model->mapTheme()->settings()->propertyAvailable( renderPlugin->nameId(), propertyAvailable );
+        bool propertyValue = false;
+        m_model->mapTheme()->settings()->propertyValue( renderPlugin->nameId(), propertyValue );
+
+        if ( propertyAvailable ) {
+            renderPlugin->setVisible( propertyValue );
+        }
     }
 
-    m_placemarkLayout.setShowCities( q->showCities() );
-    m_placemarkLayout.setShowTerrain( q->showTerrain() );
-    m_placemarkLayout.setShowOtherPlaces( q->showOtherPlaces() );
-    m_placemarkLayout.setShowLandingSites( q->propertyValue("landingsites") );
-    m_placemarkLayout.setShowCraters( q->propertyValue("craters") );
-    m_placemarkLayout.setShowMaria( q->propertyValue("maria") );
-
-    m_placemarkLayout.setDefaultLabelColor( m_model->mapTheme()->map()->labelColor() );
-    m_placemarkLayout.requestStyleReset();
-
-    m_layerManager.syncViewParamsAndPlugins( m_model->mapTheme() );
-
     emit q->themeChanged( m_model->mapTheme()->head()->mapThemeId() );
 }
 
@@ -966,9 +943,10 @@ void MarbleMap::setShowCompass( bool visible )
 
 void MarbleMap::setShowAtmosphere( bool visible )
 {
-    d->m_layerManager.removeLayer( &d->m_atmosphereLayer );
-    if ( visible ) {
-        d->m_layerManager.addLayer( &d->m_atmosphereLayer );
+    foreach ( RenderPlugin *plugin, renderPlugins() ) {
+        if ( plugin->nameId() == "atmosphere" ) {
+            plugin->setVisible( visible );
+        }
     }
 
     d->m_viewParams.setShowAtmosphere( visible );
@@ -989,7 +967,6 @@ void MarbleMap::setShowCrosshairs( bool visible )
 void MarbleMap::setShowClouds( bool visible )
 {
     d->m_viewParams.setShowClouds( visible );
-    d->m_textureLayer.setNeedsUpdate();
 
     setPropertyValue( "clouds_data", visible );
 }
@@ -1062,15 +1039,11 @@ void MarbleMap::setShowOtherPlaces( bool visible )
 void MarbleMap::setShowRelief( bool visible )
 {
     setPropertyValue( "relief", visible );
-    // Update texture map during the repaint that follows:
-    d->m_textureLayer.setNeedsUpdate();
 }
 
 void MarbleMap::setShowIceLayer( bool visible )
 {
     setPropertyValue( "ice", visible );
-    // Update texture map during the repaint that follows:
-    d->m_textureLayer.setNeedsUpdate();
 }
 
 void MarbleMap::setShowBorders( bool visible )
@@ -1086,8 +1059,6 @@ void MarbleMap::setShowRivers( bool visible )
 void MarbleMap::setShowLakes( bool visible )
 {
     setPropertyValue( "lakes", visible );
-    // Update texture map during the repaint that follows:
-    d->m_textureLayer.setNeedsUpdate();
 }
 
 void MarbleMap::setShowFrameRate( bool visible )
@@ -1095,6 +1066,11 @@ void MarbleMap::setShowFrameRate( bool visible )
     d->m_showFrameRate = visible;
 }
 
+void MarbleMap::setShowRuntimeTrace( bool visible )
+{
+    d->m_layerManager.setShowRuntimeTrace( visible );
+}
+
 void MarbleMap::setShowBackground( bool visible )
 {
     d->m_layerManager.setShowBackground( visible );
@@ -1114,7 +1090,7 @@ void MarbleMap::notifyMouseClick( int x, int y )
 
 void MarbleMap::clearVolatileTileCache()
 {
-    d->m_textureLayer.update();
+    d->m_textureLayer.reset();
     mDebug() << "Cleared Volatile Cache!";
 }
 
@@ -1124,12 +1100,6 @@ void MarbleMap::setVolatileTileCacheLimit( quint64 kilobytes )
     d->m_textureLayer.setVolatileCacheLimit( kilobytes );
 }
 
-
-bool MarbleMap::mapCoversViewport()
-{
-    return d->m_viewport.mapCoversViewport();
-}
-
 AngleUnit MarbleMap::defaultAngleUnit() const
 {
     if ( GeoDataCoordinates::defaultNotation() == GeoDataCoordinates::Decimal ) {
@@ -1157,7 +1127,7 @@ QFont MarbleMap::defaultFont() const
 void MarbleMap::setDefaultFont( const QFont& font )
 {
     GeoDataFeature::setDefaultFont( font );
-    d->m_placemarkLayout.requestStyleReset();
+    d->m_placemarkLayer.requestStyleReset();
 }
 
 QList<RenderPlugin *> MarbleMap::renderPlugins() const
@@ -1201,13 +1171,8 @@ void MarbleMap::removeLayer( LayerInterface *layer )
     d->m_layerManager.removeLayer(layer);
 }
 
-MeasureTool *MarbleMap::measureTool()
-{
-    return &d->m_measureTool;
-}
-
 // this method will only temporarily "pollute" the MarbleModel class
-TextureLayer* MarbleMap::textureLayer()
+const TextureLayer *MarbleMap::textureLayer() const
 {
     return &d->m_textureLayer;
 }
